# Copyright (C) 2007-2008 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from xml.dom import minidom

from LuciSyslog import get_logger
from LuciZope import GetReqVars
from ricci_communicator import RicciCommunicator
from conga_constants import LUCI_DEBUG_MODE
from LuciClusterInfo import LuciExtractCluModel

luci_log = get_logger()

def write_xml_resp(request, xml_obj):
	request.RESPONSE.setHeader('Content-Type', 'text/xml; charset=UTF-8')
	request.RESPONSE.setHeader('Cache-Control', 'no-cache, no-store, private')
	request.RESPONSE.write(str(xml_obj.toprettyxml()))

def result_to_xml(result):
	import types

	numeric_types = (
		types.IntType, types.BooleanType, types.LongType, types.FloatType
	)

	root = minidom.Document()

	def pyobj_to_xml(element_name, element, parent_node):
		if type(element) is types.DictType:
			if len(element) > 0:
				xml_elem = root.createElement('dict')
				xml_elem.setAttribute('name', str(element_name))

				for i in element.iterkeys():
					pyobj_to_xml(i, element[i], xml_elem)
			else:
				xml_elem = None
		elif type(element) in ( types.ListType, types.TupleType ):
			if len(element) > 0:
				xml_elem = root.createElement('list')
				xml_elem.setAttribute('name', str(element_name))
				for i in element:
					pyobj_to_xml(element_name, i, xml_elem)
			else:
				xml_elem = None
		else:
			cur_tagname = None
			try:
				if parent_node.tagName == 'list':
					cur_tagname = parent_node.getAttribute('name')
			except:
				cur_tagname = None

			if not cur_tagname:
				xml_elem = root.createElement('var')
			else:
				xml_elem = root.createElement(cur_tagname)

			if type(element) in types.StringTypes:
				cur_type = 'str'
			elif type(element) in numeric_types:
				cur_type = 'num'
			else:
				cur_type = None

			if cur_type:
				try:
					if parent_node.tagName == 'dict':
						xml_elem.setAttribute('name', str(element_name))
				except:
					pass

				xml_elem.setAttribute('type', cur_type)
				xml_elem.setAttribute('value', str(element))
			else:
				xml_elem = None

		if xml_elem is not None:
			parent_node.appendChild(xml_elem)

	pyobj_to_xml('result', result[1], root)
	res_elem = root.createElement('result')
	res_elem.setAttribute('name', 'success')
	res_elem.setAttribute('value', str(result[0]).lower())
	root.firstChild.appendChild(res_elem)
	return root

def write_err_async(request, err_msg):
	xml_obj = result_to_xml((False, { 'errors': err_msg }))
	write_xml_resp(request, xml_obj)

def get_cluster_nodes_async(self, request):
	from LuciClusterInfo import getClusterConfNodes
	from RicciQueries import getClusterConf

	fvars = GetReqVars(request, [ 'QUERY_STRING', '__SYSTEM0' ])
	if fvars['QUERY_STRING'] is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GCNA0: No query string was given')
		write_err_async(request, 'No node names were given')
		return None

	conf_node_pair = fvars['__SYSTEM0']
	if conf_node_pair is not None and len(conf_node_pair) == 2:
		conf_node = conf_node_pair[0]
		conf_node_passwd = conf_node_pair[1]
	else:
		conf_node = None
		conf_node_passwd = None

	try:
		nodes = fvars['QUERY_STRING'].split('&')
		node_list = map(lambda x: x[1], filter(lambda x: x[0][:4] == 'node', map(lambda x: x.split('='), nodes)))
		if not node_list or len(node_list) < 1:
			raise Exception, 'No node list'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GCNA: %r %s' % (e, str(e)))
		write_err_async(request, 'No node names were given')
		return None

	errors = list()
	ret = {}
	for node_host in node_list:
		try:
			if node_host == conf_node:
				rc = RicciCommunicator(node_host, enforce_trust=False)
				rc.trust()
				rc.auth(conf_node_passwd)
				if not rc.authed():
					errors.append('Authentication to node %s failed' % conf_node)
					break
			else:
				rc = RicciCommunicator(node_host)
			cluster_name = rc.cluster_info()[0]
			if not cluster_name:
				errors.append('%s is not a member of a cluster' \
					% cluster_name)
				continue
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('GCNA1: ricci: %s: %r %s' \
					% (node_host, e, str(e)))
			errors.append('Unable to communicate with the ricci agent on %s: %s' % (node_host, str(e)))
			continue

		try:
			conf = getClusterConf(rc)
			node_names = getClusterConfNodes(conf)
			if not node_names or len(node_names) < 1:
				raise Exception, 'no nodes'
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('GCNA2: ricci: %s: %r %s' \
					% (node_host, e, str(e)))
			errors.append('Unable to retrieve a list of cluster nodes from %s: %s' % (node_host, str(e)))
			continue

		ret[cluster_name] = {
			'cluster': cluster_name,
			'num_nodes': len(node_names),
			'clusternode': node_names
		}

	ret['errors'] = errors
	xml_obj = result_to_xml((len(errors) < len(node_list), ret))
	write_xml_resp(request, xml_obj)

def check_inactive_async(self, request):
	from LuciZope import user_check_inactive
	from conga_constants import INACTIVITY_TIMEOUT_SEC
	(inactive, diff) = user_check_inactive(self, request)
	xml_obj = result_to_xml((True, { 'inactive': inactive, 'diff': diff, 'max_inactive': INACTIVITY_TIMEOUT_SEC }))
	write_xml_resp(request, xml_obj)

def update_inactive_async(self, request):
	from LuciZope import user_update_inactive
	user_update_inactive(self, request)
	xml_obj = result_to_xml((True, { 'result': True }))
	write_xml_resp(request, xml_obj)

def get_sysinfo_async(self, request):
	from HelperFunctions import get_system_info

	fvars = GetReqVars(request, [ 'QUERY_STRING' ])
	query_str = fvars['QUERY_STRING']

	try:
		nodes = query_str.split('&')
		node_list = map(lambda x: (x[1], None), filter(lambda x: x[0][:4] == 'node', map(lambda x: x.split('='), nodes)))
		if not node_list or len(node_list) < 1:
			raise Exception, 'No node list'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GSIA: %r %s' % (e, str(e)))
		write_err_async(request, 'No node names were given')
		return None

	ret = {}
	try:
		ret = get_system_info(self, node_list)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GSIA1: %r %s' % (e, str(e)))
		write_err_async(request, 'Error retrieving information: %s' % str(e))
		return None

	xml_obj = result_to_xml((True, {'clusternodes':ret}))
	write_xml_resp(request, xml_obj)

def validate_auth_async(self, request):
	from LuciValidation import validate_auth
	ret = validate_auth(self, request)
	xml_obj = result_to_xml(ret)
	write_xml_resp(request, xml_obj)

def validate_clusvc_async(self, request):
	from LuciValidation import validate_clusvc_add
	model = LuciExtractCluModel(self, request)
	ret = validate_clusvc_add(model, request)
	xml_obj = result_to_xml(ret)
	write_xml_resp(request, xml_obj)
